//------------------------------------------------------------------------------
unit MyClasses;
//------------------------------------------------------------------------------
interface
//------------------------------------------------------------------------------
uses
  Classes, ScktComp, SysUtils, ComCtrls;
//------------------------------------------------------------------------------
type
//------------------------------------------------------------------------------
  THub = class(TObject)
  public
    Host: string;
    Port: integer;
    Nick, Pass: string;
    Share, Slots, Client: string;
    Enabled: boolean;
    BcFrom, BcTo: boolean;
    Tcp: TClientSocket;
    constructor Create;
    destructor Destroy; override;
  end;
//------------------------------------------------------------------------------
  TIrc = class(TObject)
  public
    Host: string;
    Port: integer;
    Nick, Pass: string;
    Ident, Descr, Channels: string;
    Enabled: boolean;
    BcFrom, BcTo: boolean;
    Tcp: TClientSocket;
    constructor Create;
    destructor Destroy; override;
  end;
//------------------------------------------------------------------------------
  THubsList = class(TList)
  private
    function Get(Index: Integer): THub;
    procedure Put(Index: Integer; const Value: THub);
  public
    ListView: TListView;
    property Items[Index: Integer]: THub read Get write Put; default;
    constructor Create;
    function Add(Item: Pointer): Integer;
    procedure Clear; override;
    procedure Delete(Index: Integer);
  end;
//------------------------------------------------------------------------------
  TIrcsList = class(TList)
  private
    function Get(Index: Integer): TIrc;
    procedure Put(Index: Integer; const Value: TIrc);
  public
    ListView: TListView;
    property Items[Index: Integer]: TIrc read Get write Put; default;
    constructor Create;
    function Add(Item: Pointer): Integer;
    procedure Clear; override;
    procedure Delete(Index: Integer);
  end;
//------------------------------------------------------------------------------
implementation
//------------------------------------------------------------------------------
{ THub }
//------------------------------------------------------------------------------
constructor THub.Create;
begin
  Tcp := TClientSocket.Create(nil);
end;
//------------------------------------------------------------------------------
destructor THub.Destroy;
begin
  Tcp.Free;
  inherited;
end;
//------------------------------------------------------------------------------
{ TIrc }
//------------------------------------------------------------------------------
constructor TIrc.Create;
begin
  Tcp := TClientSocket.Create(nil);
end;
//------------------------------------------------------------------------------
destructor TIrc.Destroy;
begin
  Tcp.Free;
  inherited;
end;
//------------------------------------------------------------------------------
{ THubsList }
//------------------------------------------------------------------------------
constructor THubsList.Create;
begin
  ListView := nil;
end;
//------------------------------------------------------------------------------
function THubsList.Get(Index: Integer): THub;
begin
  Result := inherited Get(Index);
end;
//------------------------------------------------------------------------------
procedure THubsList.Put(Index: Integer; const Value: THub);
begin
  inherited Put(Index, Value);
end;
//------------------------------------------------------------------------------
function THubsList.Add(Item: Pointer): Integer;
begin
  Result := inherited Add(Item);
  if ListView <> nil then
    with ListView.Items.Add do
    begin
      Caption := THub(Item).Host + ':' + IntToStr(THub(Item).Port);
      SubItems.Add(THub(Item).Nick);
      SubItems.Add(THub(Item).Share);
      SubItems.Add(THub(Item).Slots);
      SubItems.Add(THub(Item).Client);
      Checked := THub(Item).Enabled;
      Data := Item;
    end;
end;
//------------------------------------------------------------------------------
procedure THubsList.Clear;
begin
  inherited;
  if ListView <> nil then
    ListView.Clear;
end;
//------------------------------------------------------------------------------
procedure THubsList.Delete(Index: Integer);
begin
  if Items[Index] <> nil then
    THub(Items[Index]).Free;
  inherited;
end;
//------------------------------------------------------------------------------
{ TIrcsList }
//------------------------------------------------------------------------------
constructor TIrcsList.Create;
begin
  ListView := nil;
end;
//------------------------------------------------------------------------------
function TIrcsList.Get(Index: Integer): TIrc;
begin
  Result := inherited Get(Index);
end;
//------------------------------------------------------------------------------
procedure TIrcsList.Put(Index: Integer; const Value: TIrc);
begin
  inherited Put(Index, Value);
end;
//------------------------------------------------------------------------------
function TIrcsList.Add(Item: Pointer): Integer;
begin
  Result := inherited Add(Item);
  if ListView <> nil then
    with ListView.Items.Add do
    begin
      Caption := TIrc(Item).Host + ':' + IntToStr(TIrc(Item).Port);
      SubItems.Add(TIrc(Item).Nick);
      SubItems.Add(TIrc(Item).Ident);
      SubItems.Add(TIrc(Item).Descr);
      SubItems.Add(TIrc(Item).Channels);
      Checked := TIrc(Item).Enabled;
      Data := Item;
    end;
end;
//------------------------------------------------------------------------------
procedure TIrcsList.Clear;
var
  i: integer;
begin
  inherited;
  if ListView <> nil then
  begin
    for i := 0 to ListView.Items.Count-1 do
    begin
      TIrc(Items[i]).Free;
      Items[i] := nil;
    end;
    ListView.Clear;
  end;
end;
//------------------------------------------------------------------------------
procedure TIrcsList.Delete(Index: Integer);
begin
  if Items[Index] <> nil then
    TIrc(Items[Index]).Free;
  inherited;
end;
//------------------------------------------------------------------------------
end.
