////////////////////////////////////////////////////////////////////////////////
//
// Project name: MyIrcDcLinks
// Author: burek
// E-mail: burek021@gmail.com
// Date: 10/Oct/2009
// Version: 1.4
// Target:	Win32, Delphi 7
// License: GNU/GPL
//
// Description:
//
//   MyIrcDcLinks is an application written in Delphi 7 (for Windows), that is
//   able to connect to multiple dc hubs (and irc servers) and transmit chat
//   messages from each one to the others. The similar project is DC hublink:
//   http://dc-hublink.sourceforge.net/
//
////////////////////////////////////////////////////////////////////////////////
unit SystemTrayIconForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShellAPI, Menus;

const
  NIF_INFO = $10;

//  NIIF_INFO = $00000001;
//  NIIF_WARNING = $00000002;
//  NIIF_ERROR = $00000003;

type
  TTipType = (ttInfo, ttWarning, ttError);//NIIF_INFO, NIIF_WARNING, NIIF_ERROR);

  TDUMMYUNIONNAME    = record
    case Integer of
      0: (uTimeout: UINT);
      1: (uVersion: UINT);
  end;

  MYNOTIFYICONDATA = record
    cbSize: DWORD;
    Wnd: HWND;
    uID: UINT;
    uFlags: UINT;
    uCallbackMessage: UINT;
    hIcon: HICON;
    //Version 5.0 is 128 chars, old ver is 64 chars
    szTip: array [0..127] of Char;
    dwState: DWORD; //Version 5.0
    dwStateMask: DWORD; //Version 5.0
    szInfo: array [0..255] of Char; //Version 5.0
    DUMMYUNIONNAME: TDUMMYUNIONNAME;
    szInfoTitle: array [0..63] of Char; //Version 5.0
    dwInfoFlags: DWORD;   //Version 5.0
  end;

  TSystemTrayIconForm = class(TComponent)
  private
    { Private declarations }
    FAllowClose: boolean;
    FShowed: boolean;
    Fnid: MYNOTIFYICONDATA;
    FHandle: HWND;
    FPopupMenu: TPopupMenu;
    FWindowsMessage: Cardinal;
    FHint: string;
    FTip: string;
    FTipTitle: string;
    FTipType: TTipType;
    FIcon: TIcon;
    FTimeOut: integer;
    procedure SetAllowClose(const Value: boolean);
    procedure SetIcon(const Value: TIcon);
    procedure SetTimeOut(const Value: integer);
    procedure SetTipType(const Value: TTipType);
    procedure SetTipTitle(const Value: string);
    procedure SetTip(const Value: string);
    procedure SetHint(const Value: string);
    procedure SetPopupMenu(const Value: TPopupMenu);
    procedure WndProc(var m: TMessage);
    procedure Minimize(Sender: TObject);
    procedure Restore(Sender: TObject);
    procedure SetWindowsMessage(const Value: Cardinal);
  protected
    { Protected declarations }
    procedure Update;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure TrayAddIcon;
    procedure TrayDeleteIcon;
    procedure TrayShowTip;
  published
    { Published declarations }
    property PopupMenu: TPopupMenu read FPopupMenu write SetPopupMenu;
    property AllowClose: boolean read FAllowClose write SetAllowClose;
    property Hint: string read FHint write SetHint;
    property Tip: string read FTip write SetTip;
    property TipTitle: string read FTipTitle write SetTipTitle;
    property TipType: TTipType read FTipType write SetTipType;
    property TimeOut: integer read FTimeOut write SetTimeOut;
    property Icon: TIcon read FIcon write SetIcon;
    property TrayWndMessage: Cardinal read FWindowsMessage write SetWindowsMessage;
  end;
//------------------------------------------------------------------------------
procedure Register;
//------------------------------------------------------------------------------
implementation
//------------------------------------------------------------------------------
procedure Register;
begin
  RegisterComponents('!Mladen', [TSystemTrayIconForm]);
end;
//------------------------------------------------------------------------------
{ TSystemTrayIconForm }
//------------------------------------------------------------------------------
constructor TSystemTrayIconForm.Create(AOwner: TComponent);
begin
  inherited;
  FWindowsMessage := WM_USER;
  FHandle := Classes.AllocateHWND(WndProc);
  FTipType := ttWarning;
  FIcon := TIcon.Create;
  FShowed := false;
  if not (csDesigning in ComponentState) then
  begin
    Application.OnMinimize := Minimize;
    Application.OnRestore := Restore;
  end;
end;
//------------------------------------------------------------------------------
destructor TSystemTrayIconForm.Destroy;
begin
  inherited;
  TrayDeleteIcon;
  Classes.DeallocateHWnd(FHandle);
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.WndProc(var m: TMessage);
var
  p: TPoint;
begin
  with m do
    if Msg = FWindowsMessage then
      case LParam of
        WM_RBUTTONUP:
          begin
            SetForegroundWindow(FHandle);
            GetCursorPos(P);
            if Assigned(PopupMenu) then
            PopupMenu.Popup(P.x, P.y);
          end;
        WM_LBUTTONUP:
          Application.Restore;
      end
    else
      Result := DefWindowProc(FHandle, Msg, wParam, lParam)
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.Minimize(Sender: TObject);
begin
  TrayAddIcon;
  if not FShowed then
  begin
    FShowed := true;
    TrayShowTip;
  end;
  ShowWindow(Application.Handle, SW_HIDE);
  ShowWindow(TCustomForm(Owner).Handle, SW_HIDE);
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.Restore(Sender: TObject);
begin
  ShowWindow(Application.Handle, SW_SHOW);
  ShowWindow(TCustomForm(Owner).Handle, SW_SHOW);
  SetForegroundWindow(TCustomForm(Owner).Handle);
  TrayDeleteIcon;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.TrayAddIcon;
var
  pcHint: PChar;
begin
  if FHint = '' then
    pcHint := PChar(Application.Title)
  else
    pcHint := PChar(FHint);

  with Fnid do
  begin
    cbSize := SizeOf(Fnid);
    Wnd    := FHandle;
    uID    := 0;
    uFlags := NIF_ICON or NIF_MESSAGE or NIF_TIP;
    uCallbackMessage := FWindowsMessage;

    if FIcon.Empty then
      hIcon := Application.Icon.Handle
    else
      hIcon := FIcon.Handle;

    StrLCopy(szTip, pcHint, SizeOf(szTip) - 1);
  end;
  Shell_NotifyIcon(NIM_ADD, @Fnid);
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.TrayDeleteIcon;
begin
  with Fnid do
  begin
    cbSize := SizeOf(Fnid);
    Wnd := FHandle;
    uID := 0;
  end;
  Shell_NotifyIcon(NIM_DELETE, @Fnid);
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.TrayShowTip;
var
  pcTip, pcTipTitle: PChar;
begin
  if FTip = '' then
    pcTip := PChar(Application.Title)
  else
    pcTip := PChar(FTip);

  if FTipTitle = '' then
    pcTipTitle := 'I''m here ;)'
  else
    pcTipTitle := PChar(FTipTitle);

  with Fnid do
  begin
    cbSize := SizeOf(Fnid);
    Wnd    := FHandle;
    uID    := 0;
    uFlags := NIF_INFO;
    DUMMYUNIONNAME.uTimeout := FTimeOut;
    dwInfoFlags := Ord(FTipType); // NIIF_INFO or NIIF_ERROR or NIIF_WARNING
    StrLCopy(szInfoTitle, pcTipTitle, SizeOf(szInfoTitle) - 1);
    StrLCopy(szInfo, pcTip, SizeOf(szInfo) - 1);
  end;
  Shell_NotifyIcon(NIM_MODIFY, @Fnid);
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetAllowClose(const Value: boolean);
begin
  FAllowClose := Value;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetIcon(const Value: TIcon);
begin
  FIcon.Assign(Value);
  Update;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetPopupMenu(const Value: TPopupMenu);
begin
  FPopupMenu := Value;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetTimeOut(const Value: integer);
begin
  FTimeOut := Value;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetHint(const Value: string);
begin
  FHint := Value;
  Update;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetTip(const Value: string);
begin
  FTip := Value;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetTipTitle(const Value: string);
begin
  FTipTitle := Value;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetTipType(const Value: TTipType);
begin
  FTipType := Value;
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.Update;
var
  pcHint: PChar;
begin
  if FHint = '' then
    pcHint := PChar(Application.Title)
  else
    pcHint := PChar(FHint);

  with Fnid do
  begin
    cbSize := SizeOf(Fnid);
    Wnd    := FHandle;
    uID    := 0;
    uFlags := NIF_ICON or NIF_MESSAGE or NIF_TIP;
    uCallbackMessage := FWindowsMessage;

    if FIcon.Empty then
      hIcon := Application.Icon.Handle
    else
      hIcon := FIcon.Handle;

    StrLCopy(szTip, pcHint, SizeOf(szTip) - 1);
  end;
  Shell_NotifyIcon(NIM_MODIFY, @Fnid);
end;
//------------------------------------------------------------------------------
procedure TSystemTrayIconForm.SetWindowsMessage(const Value: Cardinal);
begin
  FWindowsMessage := Value;
end;
//------------------------------------------------------------------------------
end.
