////////////////////////////////////////////////////////////////////////////////
//
// Project name: MyIrcDcLinks
// Author: burek
// E-mail: burek021@gmail.com
// Date: 10/Oct/2009
// Version: 1.4
// Target:	Win32, Delphi 7
// License: GNU/GPL
//
// Description:
//
//   MyIrcDcLinks is an application written in Delphi 7 (for Windows), that is
//   able to connect to multiple dc hubs (and irc servers) and transmit chat
//   messages from each one to the others. The similar project is DC hublink:
//   http://dc-hublink.sourceforge.net/
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
unit UnitDC;
//------------------------------------------------------------------------------
interface
//------------------------------------------------------------------------------
uses
  Classes, ScktComp, SysUtils, ComCtrls, ExtCtrls, UnitIRCDCBase;
type
//------------------------------------------------------------------------------
  THub = class(TIrcHubBase)
  protected
    procedure OnRead(Sender: TObject; Socket: TCustomWinSocket); override;
    function EncodeLock(lock: string): string;
    procedure EscapeDCN(var s: string);
  public
    Share, Slots, Client: string;
    constructor Create; override;
    procedure SendMsgToPM(tonick, msg: string; QuickSend: boolean = false); override;
    procedure SendMsgToChat(msg: string; QuickSend: boolean = true); override;
  end;
//------------------------------------------------------------------------------
  THubsList = class(TList)
  protected
    function Get(Index: Integer): THub;
    procedure Put(Index: Integer; const Value: THub);
  public
    property Items[Index: Integer]: THub read Get write Put; default;
    procedure AddHub(HostPort, Nick, Pass, Prefix, Share, Slots, Client, Enabled, BcFrom, BcTo: string);
    procedure DeleteHub(Hub: THub);
  end;
const
  DL = '|'; // delimiter
//------------------------------------------------------------------------------
implementation
//------------------------------------------------------------------------------
uses UnitMain, UnitLocalCommands, UnitUtils, UnitMessages;
//------------------------------------------------------------------------------
{ THubsList }
//------------------------------------------------------------------------------
function THubsList.Get(Index: Integer): THub;
begin
  Result := inherited Get(Index);
end;
//------------------------------------------------------------------------------
procedure THubsList.Put(Index: Integer; const Value: THub);
begin
  inherited Put(Index, Value);
end;
//------------------------------------------------------------------------------
procedure THubsList.AddHub(HostPort, Nick, Pass, Prefix, Share, Slots, Client, Enabled, BcFrom, BcTo: string);
var
  hub: THub;
begin
  hub := THub.Create;
  GetHostPort(HostPort, hub.Host, hub.Port, 411);
  hub.Nick := Nick;
  hub.Pass := Pass;
  hub.Prefix := Prefix;
  hub.Share := Share;
  hub.Slots := Slots;
  hub.Client := Client;
  hub.Enabled := StrToBool(Enabled);
  hub.BcFrom := StrToBool(BcFrom);
  hub.BcTo := StrToBool(BcTo);
  Add(hub);
end;
//------------------------------------------------------------------------------
procedure THubsList.DeleteHub(Hub: THub);
begin
  if IndexOf(Hub) >= 0 then
    Delete(IndexOf(Hub));
  Hub.Free;
end;
//------------------------------------------------------------------------------
{ THub }
//------------------------------------------------------------------------------
procedure TrimTrailingDL(var s: string);
const
  DL = '|'; // delimiter
begin
  while (Length(s) > 0) and (s[Length(s)] = DL) do
    Delete(s, Length(s), 1);
end;
//------------------------------------------------------------------------------
procedure GetCmdParams(line: string; var cmd, params: string);
var
  i: integer;
begin
  cmd := ''; params := '';
  TrimTrailingDL(line); // remove delimiter
  if line = '' then Exit; // error

  if line[1] = '$' then // command
  begin
    i := Pos(' ', line);
    if i <= 0 then
      i := Length(line)+1;
    if i <= 0 then Exit; // error
    cmd := UpperCase(Copy(line, 2, i-2)); Delete(line, 1, i);
  end;

  params := line;
end;
//------------------------------------------------------------------------------
procedure THub.OnRead(Sender: TObject; Socket: TCustomWinSocket);
var
  i, crpos: integer;
  line, cmd, anick, params, msg_format :string;
begin
  while Socket.ReceiveLength > 0 do
    Buf := Buf + Socket.ReceiveText;

  while Pos(DL, Buf) > 0 do // only consider complete nmdc messages
  begin
    crpos := Pos(DL, Buf);
    line := Copy(Buf, 1, crpos);
    Delete(Buf, 1, crpos); // remove this line from buffer
    GetCmdParams(line, cmd, params);

    // *************************************************************************
    // $Lock
    // *************************************************************************
    if cmd = 'LOCK' then
    begin
      params := Copy(params, 1, Pos(' ', params) - 1);
//      Socket.SendText(Format('$Supports UserCommand NoGetINFO NoHello UserIP2TTHSearch GetZBlock |$Key %s|$ValidateNick %s|', [EncodeLock(params), Nick]));
//      Socket.SendText(Format('$Supports NoGetINFO |$Key %s|$ValidateNick %s|', [EncodeLock(params), Nick]));
//      Socket.SendText(Format('$Supports NoGetINFO |$Key %s|$ValidateNick %s|', [EncodeLock(params), Nick]));
      Socket.SendText(Format('$Supports |$Key %s|$ValidateNick %s|', [EncodeLock(params), Nick]));
      continue;
    end;

    // *************************************************************************
    // $GetPass
    // *************************************************************************
    if cmd = 'GETPASS' then
    begin
      Socket.SendText(Format('$MyPass %s|', [Pass]));
      continue;
    end;

    // *************************************************************************
    // $Hello
    // *************************************************************************
    if cmd = 'HELLO' then
    begin
      if params <> Nick then // error
        continue;
//      Socket.SendText(Format('$Version 1,0091|$GetNickList|$MyINFO $ALL %s <%s,M:A,H:0/1/0,S:%s>$ $100'#1'$$%s$|', [Nick, Client, Slots, Share]));
      Socket.SendText(Format('$Version 1,0091|$MyINFO $ALL %s <%s,M:A,H:0/1/0,S:%s>$ $100'#1'$$%s$|', [Nick, Client, Slots, Share]));
//        Socket.SendText(Format('$Version 1,0091|$GetNickList|$MyINFO $ALL %s <%s,M:A,H:0/1/0,S:%s>$ $100'#1'$$%s$|', [Nick, Client, Slots, Share]));
//        Socket.SendText(Format('$Version 1,0091|$MyINFO $ALL %s <%s,M:A,H:0/1/0,S:%s>$ $100'#1'$$%s$|', [Nick, Client, Slots, Share]));
      continue;
    end;

    // *************************************************************************
    // $LogedIn
    // *************************************************************************
    if cmd = 'LOGEDIN' then
    begin
      if params <> Nick then // error
        continue;
//      Socket.SendText(Format('$Version 1,0091|$GetNickList|$MyINFO $ALL %s <%s,M:A,H:0/1/0,S:%s>$ $100'#1'$$%s$|', [Nick, Client, Slots, Share]));
      continue;
    end;

    // *************************************************************************
    // $OpList
    // *************************************************************************
    if cmd = 'OPLIST' then
    begin
      if ConnStep = 0 then
        Tcp.Socket.SendText(Format('<%s> %s|', [Nick, MSG_ADVERT]));
      OpList.Text := Trim(StringReplace(params, '$$', #13#10, [rfReplaceAll]));
      ConnStep := 1;
      continue;
    end;


    // *************************************************************************
    // $LogedIn
    // *************************************************************************
    if cmd = 'SEARCH' then
    begin
      ConnStep := 1;
      continue;
    end;

    if ConnStep < 1 then
      continue;

    // *************************************************************************
    // $To:
    // *************************************************************************
    if cmd = 'TO:' then
    begin
      // $To: burek From: burek $<burek> asdsad asdsad
      i := Pos('From: ', params);
      if i <= 0 then continue; // error
      Delete(params, 1, i+5);

      i := Pos(' ', params);
      if i <= 0 then continue; // error
      anick := Copy(params, 1, i-1);
      Delete(params, 1, i);

      i := Pos('> ', params); // get the command
      if i <= 0 then continue; // error
      Delete(params, 1, i+1);

      if Pos('.', params) <> 1 then // not regular pm
        continue;
      i := Pos(' ', params); // get the command
      if i <= 0 then // cmd with no parameters
        i := Length(params)+1;
      cmd := UpperCase(Copy(params, 2, i-2));
      Delete(params, 1, Length(cmd) + 2);
      ParseLocalCommands(Self, anick, cmd, params);
      continue; // don't broadcast link commands
    end;

    if cmd <> '' then // we are not interested in commands, only chat
      continue;


    // *************************************************************************
    // Message
    // *************************************************************************
    if Pos('<', params) = 1 then
    begin // remove <> around the nick
      Delete(params, 1, 1);
      if (Pos(Nick, params) = 1) or MainForm.HasLinkPrefix(params) then // own message
        continue;
      i := Pos('>', params);
      if i <= 0 then continue; // error
      anick := Copy(params, 1, i-1);
      params := Copy(params, i+2, MaxInt);
      msg_format := '%s: %s';

      if Pos('/me ', params) = 1 then // /me
      begin
        Delete(params, 1, 4);
        msg_format := '* %s %s';
      end;

      if Pos('.', params) = 1 then // .commands
      begin
        i := Pos(' ', params);
        if i <= 0 then // cmd with no parameters
          i := Length(params)+1;
        cmd := UpperCase(Copy(params, 2, i-2));
        Delete(params, 1, Length(cmd) + 2);
        ParseLocalCommands(Self, anick, cmd, params);
        continue; // don't broadcast link commands
      end;
    end

    else if Pos('** ', params) = 1 then // +me
    begin
      Delete(params, 1, 3);
      i := Pos(' ', params);
      if i <= 0 then continue; // error
      anick := Copy(params, 1, i-1);
      params := Copy(params, i+1, MaxInt);
      msg_format := '* %s %s';
    end

    else if Pos('* ', params) = 1 then // action
    begin
      Delete(params, 1, 2);
      if (Pos(Nick, params) = 1) or MainForm.HasLinkPrefix(params) then // own message
        continue;
      i := Pos(' ', params);
      if i <= 0 then continue; // error
      anick := Copy(params, 1, i-1);
      params := Copy(params, i+1, MaxInt);
      msg_format := '* %s %s';
    end

    ;

    if BcFrom and not Muted then
      MainForm.Broadcast(Self, Prefix, anick, params, msg_format);
  end;
end;
//------------------------------------------------------------------------------
function THub.EncodeLock(lock :string): string;
var
  i, len :integer;
begin
  // get size from lock-string
  len := Length(lock);

  // create buffer for key
  SetLength(Result, len);

  // create the key-data
  for i := 2 to len do
    Result[i] := Chr(Ord(lock[i]) xor Ord(lock[i-1]));

  Result[1] := Chr(Ord(lock[1]) xor Byte(lock[len]) xor Ord(lock[len-1]) xor 5);

  // nibble-swap the key
  for i := 1 to len do
     Result[i] := Chr(((Ord(Result[i]) shl 4) and $F0) or ((Ord(Result[i]) shr 4) and $0F));

  escapeDCN(Result);
end;
//------------------------------------------------------------------------------
procedure THub.EscapeDCN(var s :string);
begin
  MyReplace(s, #0, '/%DCN000%/');
  MyReplace(s, #5, '/%DCN005%/');
  MyReplace(s, #36, '/%DCN036%/');
  MyReplace(s, #96, '/%DCN096%/');
  MyReplace(s, #124, '/%DCN124%/');
  MyReplace(s, #126, '/%DCN126%/');
end;
//------------------------------------------------------------------------------
procedure THub.SendMsgToPM(tonick, msg: string; QuickSend: boolean = false);
var
  fmt: string;
begin
  fmt := Format('$To: %%s From: %s $<%s> %%s|', [Nick, Nick]); // fmt = '...%s...%s...' (tonick, msg)
  SendMsgToPMFmt(tonick, msg, fmt, QuickSend);
end;
//------------------------------------------------------------------------------
procedure THub.SendMsgToChat(msg: string; QuickSend: boolean = true);
var
  fmt: string;
begin
  fmt := Format('<%s> %%s|', [Nick]); // fmt = '...%s...' (msg)
  SendMsgToChatFmt(msg, fmt, QuickSend);
end;
//------------------------------------------------------------------------------
constructor THub.Create;
begin
  inherited;
  Tcp.OnRead := Self.OnRead;
end;
//------------------------------------------------------------------------------
end.
