////////////////////////////////////////////////////////////////////////////////
//
// Project name: MyIrcDcLinks
// Author: burek
// E-mail: burek021@gmail.com
// Date: 10/Oct/2009
// Version: 1.4
// Target:	Win32, Delphi 7
// License: GNU/GPL
//
// Description:
//
//   MyIrcDcLinks is an application written in Delphi 7 (for Windows), that is
//   able to connect to multiple dc hubs (and irc servers) and transmit chat
//   messages from each one to the others. The similar project is DC hublink:
//   http://dc-hublink.sourceforge.net/
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
unit UnitIRC;
//------------------------------------------------------------------------------
interface
//------------------------------------------------------------------------------
uses
  Classes, ScktComp, SysUtils, ComCtrls, ExtCtrls, UnitIRCDCBase;
type
//------------------------------------------------------------------------------
  TIrc = class(TIrcHubBase)
  protected
    procedure OnConnect(Sender: TObject; Socket: TCustomWinSocket); override;
    procedure OnRead(Sender: TObject; Socket: TCustomWinSocket); override;
  public
    Ident, Descr, Channels, CurrNick: string;
    constructor Create; override;
    procedure Connect; override;
    procedure SendMsgToPM(tonick, msg: string; QuickSend: boolean = false); override;
    procedure SendMsgToChat(msg: string; QuickSend: boolean = true); override;
  end;
//------------------------------------------------------------------------------
  TIrcsList = class(TList)
  private
    function Get(Index: Integer): TIrc;
    procedure Put(Index: Integer; const Value: TIrc);
  public
    property Items[Index: Integer]: TIrc read Get write Put; default;
    procedure AddIrc(HostPort, Nick, Pass, Prefix, Ident, Descr, Channels, Enabled, BcFrom, BcTo: string);
    procedure DeleteIrc(Irc: TIrc);
  end;
const
  DL = #10; // delimiter
  NL = #13#10; // new line (CRLF)
//------------------------------------------------------------------------------
implementation
uses UnitMain, UnitLocalCommands, UnitUtils, UnitMessages;
//------------------------------------------------------------------------------
{ TIrcsList }
//------------------------------------------------------------------------------
function TIrcsList.Get(Index: Integer): TIrc;
begin
  Result := inherited Get(Index);
end;
//------------------------------------------------------------------------------
procedure TIrcsList.Put(Index: Integer; const Value: TIrc);
begin
  inherited Put(Index, Value);
end;
//------------------------------------------------------------------------------
procedure TIrcsList.AddIrc(HostPort, Nick, Pass, Prefix, Ident, Descr, Channels, Enabled, BcFrom, BcTo: string);
var
  irc: TIrc;
begin
  irc := TIrc.Create;
  GetHostPort(HostPort, irc.Host, irc.Port, 6667);
  irc.Nick := Nick;
  irc.Pass := Pass;
  irc.Prefix := Prefix;
  irc.Ident := Ident;
  irc.Descr := Descr;
  irc.Channels := Channels;
  irc.Enabled := StrToBool(Enabled);
  irc.BcFrom := StrToBool(BcFrom);
  irc.BcTo := StrToBool(BcTo);
  Add(irc);
end;
//------------------------------------------------------------------------------
procedure TIrcsList.DeleteIrc(Irc: TIrc);
begin
  if IndexOf(Irc) >= 0 then
    Delete(IndexOf(Irc));
  Irc.Free;
end;
//------------------------------------------------------------------------------
{ TIrc }
//------------------------------------------------------------------------------
procedure TIrc.Connect;
begin
  CurrNick := Nick;
  inherited;
end;
//------------------------------------------------------------------------------
procedure TIrc.OnConnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  inherited;
  Socket.SendText(Format('NICK %s'#13#10'USER %s "" "%s" :%s'#13#10, [CurrNick, Ident, Host, Descr]));
end;
//------------------------------------------------------------------------------
procedure GetNickIdentHost(mask: string; var nick, ident, host: string);
var
  i: integer;
begin
  // burek!burek@512855F.BDACAB.238F7411.IP
  nick := ''; ident := ''; host := '';
  if mask = '' then
    Exit;

  i := Pos('!', mask);
  if i <= 0 then Exit; // error
  nick := Copy(mask, 1, i-1); Delete(mask, 1, i);

  i := Pos('@', mask);
  if i <= 0 then Exit; // error
  ident := Copy(mask, 1, i-1); Delete(mask, 1, i);

  host := mask;
end;
//------------------------------------------------------------------------------
procedure TrimTrailingCRLF(var s: string);
begin
  while (Length(s) > 0) and (s[Length(s)] in [#10, #13]) do
    Delete(s, Length(s), 1);
end;
//------------------------------------------------------------------------------
procedure GetWhoCmdParams(line: string; var who, cmd, params: string);
var
  i: integer;
begin
  who := ''; cmd := ''; params := '';
  TrimTrailingCRLF(line); // remove CRLF
  if line = '' then Exit; // error

  if line[1] = ':' then // has WHO
  begin
    i := Pos(' ', line);
    if i <= 0 then Exit; // error
    who := Copy(line, 2, i-2); Delete(line, 1, i);
    line := TrimLeft(line); // because of RFC, there can be 1 or more spaces
  end;

  i := Pos(' ', line);
  if i <= 0 then Exit; // error
  cmd := UpperCase(Copy(line, 1, i-1)); Delete(line, 1, i);
  line := TrimLeft(line); // because of RFC, there can be 1 or more spaces

  params := line;
end;
//------------------------------------------------------------------------------
procedure GetOppedDeopped(line: string; var Opped, Deopped: TStringList);
var
  i: integer;
  modes: string;
  sign: boolean;
begin
  // #dddd +o-o+o-o pivo pivo pivo pivo
  Opped := TStringList.Create;
  Deopped := TStringList.Create;

  i := Pos(' ', line);
  if i <= 0 then Exit; // error
  Delete(line, 1, i);

  i := Pos(' ', line); // modes
  if i <= 0 then Exit;// not a nick mode
  modes := Copy(line, 1, i-1);

  Delete(line, 1, i); // line contains list of nicks
  StringReplace(line, ' ', ',', [rfReplaceAll]);

  with TStringList.Create do
  try
    CommaText := line;
    sign := true;
    i := 0;
    while Length(modes) > 0 do
    begin
      case modes[1] of
        '+': sign := true;
        '-': sign := false;
        'o', 'a', 'h', 'q':
          begin
            if i >= Count then // not enough nicks for these modes
              continue;
            if sign then
              Opped.Add(Strings[i])
            else
              Deopped.Add(Strings[i]);
            Inc(i);
          end;
        'l', 'b', 'v', 'k': Inc(i);
      end; // case
      System.Delete(modes, 1, 1);
    end;
  finally
    Free;
  end;
end;
//------------------------------------------------------------------------------
procedure TIrc.OnRead(Sender: TObject; Socket: TCustomWinSocket);
const
  OP_PREFIXES = ['~', '&', '@'];

  RPL_ENDOFMOTD           = '376';
  ERR_NOMOTD              = '422';

  WHOIS_BEGIN             = 311;
    RPL_WHOISUSER           = '311';
    RPL_WHOISCHANNELS       = '319';
    RPL_ENDOFWHOIS          = '318';
  WHOIS_END               = 319;

  ERR_ERRONEUSNICKNAME    = '432';
  ERR_NICKNAMEINUSE       = '433';

  ERR_PASSWDMISMATCH      = '464';
  BNC_ERR_PASSWORDMISSING = '701';

  ERR_YOUREBANNEDCREEP  = '465';

  // Color Codes
  CC_B = #$02; // bold
  CC_C = #$03; // color
  CC_R = #$16; // reverse
  CC_U = #$1F; // underline
var
  i, crpos: integer;
  line, who, cmd, params :string;
  anick, aident, ahost: string;
  ops, deops: TStringList;
begin
  while Socket.ReceiveLength > 0 do
    Buf := Buf + Socket.ReceiveText;

  while Pos(DL, Buf) > 0 do // only consider complete irc messages
  begin
    crpos := Pos(DL, Buf);
    line := Copy(Buf, 1, crpos);
    Delete(Buf, 1, crpos); // remove this line from buffer
    GetWhoCmdParams(line, who, cmd, params);

    // #########################################################################
    // Commands that are ALWAYS parsed
    // #########################################################################

    // *************************************************************************
    if cmd = 'PING' then
    // *************************************************************************
    begin
      Socket.SendText(StringReplace(line, 'PING', 'PONG', []));
      continue;
    end;

    // *************************************************************************
    // Is logged in?
    // *************************************************************************
    if ( (ConnStep = 0) and ((cmd = 'MODE') or (cmd = RPL_ENDOFMOTD) or (cmd = ERR_NOMOTD)) ) or
       ( (cmd = 'NOTICE') and (Pos('Pristup Vam je dozvoljen! Za ulazak u sobu', params) > 0) ) then
    begin
      ConnStep := 1; // logged in
      Socket.SendText(Format('PRIVMSG NickServ IDENTIFY %s'#13#10, [Pass]));
      Sleep(10);
      Socket.SendText(Format('JOIN %s'#13#10, [Channels]));
    end;

    // *************************************************************************
    // Must change nick
    // *************************************************************************
    if (cmd = ERR_NICKNAMEINUSE) or (cmd = ERR_ERRONEUSNICKNAME) then
    begin
      CurrNick := Nick + IntToStr(Random(9999));
      Socket.SendText(Format('NICK %s'#13#10, [CurrNick]));
      continue;
    end;

    if ConnStep < 1 then // still not logged in, ignore all messages
      continue;

    // #########################################################################
    // Commands that are parsed ONLY if logged in
    // #########################################################################

    // *************************************************************************
    if (cmd = 'PRIVMSG') then
    // *************************************************************************
    begin
      // :burek!burek@512855F.BDACAB.238F7411.IP PRIVMSG #kanal :jeje
      GetNickIdentHost(who, anick, aident, ahost);
      if Copy(params, 1, 1) = '#' then // channel messages
      begin
        params := Copy(params, Pos(':', params)+1, MaxInt);
        if Copy(params, 1, 1) = Chr(1) then begin // /me text
          params := Copy(params, 2, MaxInt);
          if Pos('ACTION', params) = 1 then
            params := StringReplace(params, 'ACTION ', '', []);
          if Copy(params, Length(params), 1) = Chr(1) then
            Delete(params, Length(params), 1);
          // Delete(params, Length(params), 1);
          if BcFrom then
            MainForm.Broadcast(Self, Prefix, anick, params, '* %s %s');
        end else // /msg nick text
          if BcFrom then
            MainForm.Broadcast(Self, Prefix, anick, params, '%s: %s');
        continue;
      end
      else // private messages
      begin
        params := Copy(params, Pos(':', params)+1, MaxInt);
        if Copy(params, 1, 1) = '.' then // local commands
        begin
          i := Pos(' ', params); // get the command
          if i <= 0 then // cmd with no parameters
            i := Length(params)+1;
          cmd := UpperCase(Copy(params, 2, i-2));
          ParseLocalCommands(Self, anick, cmd, params);
        end
        else ; // ignore other private messages
      end;
    end;

    // *************************************************************************
    if (cmd = '353') then // NAMES LIST
    // *************************************************************************
    begin
      i := Pos(':', params);
      if i <= 0 then continue; // error
      params := StringReplace(Copy(params, i+1, MaxInt), ' ', ',', [rfReplaceAll]); // list of comma-separated nicks
      with TStringList.Create do
        try
          CommaText := params;
          for i := 0 to Count-1 do
            if (Trim(Strings[i]) <> '') and (Length(Strings[i]) > 0) and (Strings[i][1] in OP_PREFIXES) then
              OpList.Add(Copy(Strings[i], 2, MaxInt));
        finally
          Free;
        end;
        continue;
    end;

    // *************************************************************************
    if (cmd = 'QUIT') then // remove all entries of that nick 
    // *************************************************************************
    begin
      GetNickIdentHost(who, anick, aident, ahost);
      while OpList.IndexOf(anick) >= 0 do
        OpList.Delete(OpList.IndexOf(anick));
      continue;
    end;

    // *************************************************************************
    if (cmd = 'JOIN') then // advertise 
    // *************************************************************************
    begin
      GetNickIdentHost(who, anick, aident, ahost);
      if anick = Nick then // my nick
        Tcp.Socket.SendText(Format('PRIVMSG %s :%s'#13#10, [Copy(params, 2, MaxInt), MSG_ADVERT]));
      continue;
    end;

    // *************************************************************************
    if (cmd = 'PART') then // remove one entry of that nick 
    // *************************************************************************
    begin
      GetNickIdentHost(who, anick, aident, ahost);
      if OpList.IndexOf(anick) >= 0 then
        OpList.Delete(OpList.IndexOf(anick));
      continue;
    end;

    // *************************************************************************
    if (cmd = 'MODE') then
    // *************************************************************************
    begin
      GetOppedDeopped(params, ops, deops);
      OpList.AddStrings(ops);
      for i := 0 to deops.Count-1 do
        if OpList.IndexOf(deops[i]) >= 0 then
          OpList.Delete(OpList.IndexOf(deops[i]));
      ops.Free;
      deops.Free;
      continue;
    end;

{
    // Track nick changes
    // *************************************************************************
    if (cmd = 'NICK') then
    // *************************************************************************
    begin
      // :burek!burek@512855F.BDACAB.238F7411.IP NICK :jeje
      GetNickIdentHost(who, anick, aident, ahost);
      params := Copy(params, 2, MaxInt); // remove ':' from ':jeje'
      // MainForm.Notification(Self, Format('%s %s %s', []));
    end;
}

    ////////////////////////////////////////////////////////////////////////////
  end;

end;
//------------------------------------------------------------------------------
procedure TIrc.SendMsgToPM(tonick, msg: string; QuickSend: boolean = false);
var
  fmt: string;
begin
  fmt := 'PRIVMSG %s :%s'#13#10; // fmt = '...%s...%s...' (tonick, msg)
  SendMsgToPMFmt(tonick, msg, fmt, QuickSend);
end;
//------------------------------------------------------------------------------
procedure TIrc.SendMsgToChat(msg: string; QuickSend: boolean = true);
var
  fmt: string;
begin
  fmt := Format('PRIVMSG %s :%%s'#13#10, [Channels]); // fmt = '...%s...' (msg)
  SendMsgToChatFmt(msg, fmt, QuickSend);
end;
//------------------------------------------------------------------------------
constructor TIrc.Create;
begin
  inherited;
  Tcp.OnConnect := OnConnect;
  Tcp.OnError := OnError;
end;
//------------------------------------------------------------------------------
end.
