////////////////////////////////////////////////////////////////////////////////
//
// Project name: MyIrcDcLinks
// Author: burek
// E-mail: burek021@gmail.com
// Date: 10/Oct/2009
// Version: 1.4
// Target:	Win32, Delphi 7
// License: GNU/GPL
//
// Description:
//
//   MyIrcDcLinks is an application written in Delphi 7 (for Windows), that is
//   able to connect to multiple dc hubs (and irc servers) and transmit chat
//   messages from each one to the others. The similar project is DC hublink:
//   http://dc-hublink.sourceforge.net/
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
unit UnitIRCDCBase;
//------------------------------------------------------------------------------
interface
//------------------------------------------------------------------------------
uses
  Classes, ScktComp, SysUtils, ComCtrls, ExtCtrls;
//------------------------------------------------------------------------------
type
  TIrcHubBase = class
  protected
    Queue: TStringList;
    TimerQueue: TTimer;
    procedure OnConnect(Sender: TObject; Socket: TCustomWinSocket); virtual;
    procedure OnDisconnect(Sender: TObject; Socket: TCustomWinSocket); virtual;
    procedure OnError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer); virtual;
    procedure OnRead(Sender: TObject; Socket: TCustomWinSocket); virtual; abstract;
    procedure TimerQueueTimer(Sender: TObject); virtual;
  public
    Host: string;
    Port: integer;
    Nick, Pass: string;
    Prefix, PrefixFormat: string;
    Enabled: boolean;
    BcFrom, BcTo: boolean;
    Tcp: TClientSocket;
    Connected: boolean;
    ConnStep: integer;
    Buf: string;
    Muted: boolean;
    OpList: TStringList;
    IgnoreList: TStringList;
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Connect; virtual;
    procedure SendMsgToPMFmt(tonick, msg, fmt: string; QuickSend: boolean = false); virtual;
    procedure SendMsgToPM(tonick, msg: string; QuickSend: boolean = false); virtual; abstract;
    procedure SendMsgToChatFmt(msg, fmt: string; QuickSend: boolean = true); virtual;
    procedure SendMsgToChat(msg: string; QuickSend: boolean = true); virtual; abstract;
    function GetMyInfo: string;
    function NickIgnored(aprefix, anick: string): boolean;
    function GetPrefixedNick(aprefix, anick: string): string;
  end;
//------------------------------------------------------------------------------
implementation
uses UnitMain, UnitUtils;
//------------------------------------------------------------------------------
{ TIrcHubBase }
//------------------------------------------------------------------------------
procedure TIrcHubBase.Connect;
begin
  Tcp.Socket.Close;
  Connected := false;
  ConnStep := 0;
  Tcp.Host := Host;
  Tcp.Port := Port;
  Tcp.Active := true;
end;
//------------------------------------------------------------------------------
constructor TIrcHubBase.Create;
begin
  Tcp := TClientSocket.Create(nil);
  Tcp.OnConnect := OnConnect;
  Tcp.OnDisconnect := OnDisconnect;
  Tcp.OnError := OnError;
  Tcp.OnRead := OnRead;
  Muted := false;
  OpList := TStringList.Create;
  IgnoreList := TStringList.Create;
  IgnoreList.Sorted := true;
  IgnoreList.Duplicates := dupIgnore;
  Queue := TStringList.Create;
  TimerQueue := TTimer.Create(nil);
  TimerQueue.Interval := 1000;
  TimerQueue.OnTimer := TimerQueueTimer;
end;
//------------------------------------------------------------------------------
destructor TIrcHubBase.Destroy;
begin
  TimerQueue.Free;
  Queue.Free;
  IgnoreList.Free;
  OpList.Free;
  Tcp.Free;
  inherited;
end;
//------------------------------------------------------------------------------
procedure TIrcHubBase.OnConnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  Connected := true;
  MainForm.RefreshItem(Self);
end;
//------------------------------------------------------------------------------
procedure TIrcHubBase.OnDisconnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  Connected := false;
  MainForm.RefreshItem(Self);
end;
//------------------------------------------------------------------------------
procedure TIrcHubBase.OnError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  ErrorCode := 0;
  Socket.Close;
end;
//------------------------------------------------------------------------------
procedure TIrcHubBase.SendMsgToChatFmt(msg, fmt: string; QuickSend: boolean = true);
var
  i: integer;
begin
  if Pos(#13#10, msg) <= 0 then
    if QuickSend then
      Tcp.Socket.SendText(Format(fmt, [msg]))
    else
      Queue.Add(Format(fmt, [msg]))
  else
    with TStringList.Create do
    try
      Text := msg;
      for i := 0 to Count-1 do
        if QuickSend then
          Tcp.Socket.SendText(Format(fmt, [Strings[i]]))
        else
          Queue.Add(Format(fmt, [Strings[i]]));
    finally
      Free;
    end;
end;
//------------------------------------------------------------------------------
procedure TIrcHubBase.SendMsgToPMFmt(tonick, msg, fmt: string; QuickSend: boolean = false);
var
  i: integer;
begin
  if Pos(#13#10, msg) <= 0 then
  begin
    if Trim(msg) = '' then
      Exit;
    if QuickSend then
      Tcp.Socket.SendText(Format(fmt, [tonick, msg]))
    else
      Queue.Add(Format(fmt, [tonick, msg]))
  end
  else
    with TStringList.Create do
    try
      Text := msg;
      for i := 0 to Count-1 do
      begin
        if Trim(Strings[i]) = '' then
          continue;
        if QuickSend then
          Tcp.Socket.SendText(Format(fmt, [tonick, Strings[i]]))
        else
          Queue.Add(Format(fmt, [tonick, Strings[i]]));
      end;
    finally
      Free;
    end;
end;
//------------------------------------------------------------------------------
procedure TIrcHubBase.TimerQueueTimer(Sender: TObject);
begin
  if Queue.Count <= 0 then
    Exit;
  Tcp.Socket.SendText(Queue[0]);
  Queue.Delete(0);
end;
//------------------------------------------------------------------------------
function TIrcHubBase.GetMyInfo(): string;
begin
  Result := Format('BroadcastFromHere: %s, BroadcastToHere: %s, Muted: %s', [YesNo(Ord(BcFrom)), YesNo(Ord(BcTo)), YesNo(Ord(Muted))]);
end;
//------------------------------------------------------------------------------
function TIrcHubBase.NickIgnored(aprefix, anick: string): boolean;
begin
  Result := IgnoreList.IndexOf(GetPrefixedNick(aprefix, anick)) >= 0;
end;
//------------------------------------------------------------------------------
function TIrcHubBase.GetPrefixedNick(aprefix, anick: string): string;
begin
  Result := Format('{%s}%s', [aprefix, anick]);
end;
//------------------------------------------------------------------------------
end.
