////////////////////////////////////////////////////////////////////////////////
//
// Project name: MyIrcDcLinks
// Author: burek
// E-mail: burek021@gmail.com
// Date: 10/Oct/2009
// Version: 1.4
// Target:	Win32, Delphi 7
// License: GNU/GPL
//
// Description:
//
//   MyIrcDcLinks is an application written in Delphi 7 (for Windows), that is
//   able to connect to multiple dc hubs (and irc servers) and transmit chat
//   messages from each one to the others. The similar project is DC hublink:
//   http://dc-hublink.sourceforge.net/
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
unit UnitMain;
//------------------------------------------------------------------------------
interface
//------------------------------------------------------------------------------
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, SystemTrayIcon, Sockets, ImgList, ExtCtrls,
  ScktComp, UnitDC, UnitIRC, Menus, IniFiles;
//------------------------------------------------------------------------------
type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    lvIrcs: TListView;
    lblIrcHost: TLabel;
    edIrcHostPort: TEdit;
    edIrcNick: TEdit;
    lblIrcNick: TLabel;
    lblIrcPass: TLabel;
    edIrcPass: TEdit;
    lblIrcIdent: TLabel;
    edIrcIdent: TEdit;
    edIrcDescription: TEdit;
    lblIrcDescription: TLabel;
    lblIrcChannels: TLabel;
    edIrcChannels: TEdit;
    btnIrcUpdate: TButton;
    lblHubHostPort: TLabel;
    edHubHostPort: TEdit;
    edHubNick: TEdit;
    lblHubNick: TLabel;
    lblHubPass: TLabel;
    edHubPass: TEdit;
    lblHubShare: TLabel;
    edHubShare: TEdit;
    edHubSlots: TEdit;
    lblHubSlots: TLabel;
    lblHubClient: TLabel;
    edHubClient: TEdit;
    btnHubUpdate: TButton;
    lvHubs: TListView;
    cbHubBcFrom: TCheckBox;
    cbHubBcTo: TCheckBox;
    cbIrcBcTo: TCheckBox;
    cbIrcBcFrom: TCheckBox;
    ilOnOffIcons: TImageList;
    TimerPoll: TTimer;
    btnConnectAll: TButton;
    TabSheet3: TTabSheet;
    mmBadWords: TMemo;
    cbHubEnabled: TCheckBox;
    cbIrcEnabled: TCheckBox;
    lblHubPrefix: TLabel;
    edHubPrefix: TEdit;
    lblIrcPrefix: TLabel;
    edIrcPrefix: TEdit;
    btnHubNew: TButton;
    btnHubDelete: TButton;
    btnIrcDelete: TButton;
    btnIrcNew: TButton;
    pmConnDisconn: TPopupMenu;
    Connect1: TMenuItem;
    Disconnect1: TMenuItem;
    btnBadWordsSave: TButton;
    btnBadWordsReload: TButton;
    cbAutoConnect: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lvHubsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure lvIrcsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure TimerPollTimer(Sender: TObject);
    procedure btnConnectAllClick(Sender: TObject);
    procedure btnHubUpdateClick(Sender: TObject);
    procedure btnIrcUpdateClick(Sender: TObject);
    procedure btnHubNewClick(Sender: TObject);
    procedure btnHubDeleteClick(Sender: TObject);
    procedure btnIrcNewClick(Sender: TObject);
    procedure btnIrcDeleteClick(Sender: TObject);
    procedure pmConnDisconnPopup(Sender: TObject);
    procedure Connect1Click(Sender: TObject);
    procedure Disconnect1Click(Sender: TObject);
    procedure btnBadWordsSaveClick(Sender: TObject);
    procedure btnBadWordsReloadClick(Sender: TObject);
    procedure cbAutoConnectClick(Sender: TObject);
  private
    { Private declarations }
    Hubs: THubsList;
    Ircs: TIrcsList;
    BadWords: TStringList;
    SystemTrayIcon1: TSystemTrayIcon;
    procedure LoadConfig;
    procedure LoadBadWords;
    procedure LoadIrcsHubs(IrcOrHub: string);
    procedure SaveIrcsHubs(IrcOrHub: string);
    procedure ConnectAll;
    procedure UpdateDisplayHubs;
    procedure UpdateDisplayIrcs;
  public
    { Public declarations }
    procedure Broadcast(Origin: Pointer; Prefix, ANick, Msg, MsgFormat: string);
    procedure RefreshItem(Origin: Pointer);
    function GetLinks: string;
    function HasBadWords(msg: string): boolean;
    function HasLinkPrefix(anick: string): boolean;
  end;
//------------------------------------------------------------------------------
const
  FILE_HUBS = 'hubs.ini';
  FILE_IRCS = 'ircs.ini';
  FILE_BADWORDS = 'badwords.ini';
var
  MainForm: TMainForm;
  BaseDir: string;
  CSNotification, CSRefreshItem, CSGetLinks: RTL_CRITICAL_SECTION;
//------------------------------------------------------------------------------
implementation
uses SyncObjs, UnitUtils, UnitIRCDCBase, UnitMessages;
//------------------------------------------------------------------------------
{$R *.dfm}
//------------------------------------------------------------------------------
procedure TMainForm.FormCreate(Sender: TObject);
begin
  SystemTrayIcon1 := TSystemTrayIcon.Create(Self);
  InitializeCriticalSection(CSNotification);
  InitializeCriticalSection(CSRefreshItem);
  InitializeCriticalSection(CSGetLinks);
  Hubs := THubsList.Create;
  Ircs := TIrcsList.Create;
  BaseDir := ExtractFilePath(Application.ExeName);
  LoadConfig;
  BadWords := TStringList.Create;
  LoadBadWords;
  mmBadWords.Lines.Assign(BadWords);
end;
//------------------------------------------------------------------------------
procedure TMainForm.FormDestroy(Sender: TObject);
begin
  SaveIrcsHubs('hubs');
  SaveIrcsHubs('ircs');
end;
//------------------------------------------------------------------------------
procedure TMainForm.LoadConfig;
begin
  Hubs.Clear;
  Ircs.Clear;
  LoadIrcsHubs('hubs');
  LoadIrcsHubs('ircs');
end;
//------------------------------------------------------------------------------
procedure TMainForm.LoadIrcsHubs(IrcOrHub: string);
var
  fname: string;
  i: integer;
  ini: TIniFile;
  sections, section: TStringList;
begin
  IrcOrHub := LowerCase(IrcOrHub);
  if IrcOrHub = 'hubs' then
    fname := BaseDir + FILE_HUBS
  else
    fname := BaseDir + FILE_IRCS;
  if not FileExists(fname) then
    Exit;

  ini := TIniFile.Create(fname);
  sections := TStringList.Create;
  section := TStringList.Create;
  try
    ini.ReadSections(sections);
    for i := 0 to sections.Count-1 do
    begin
      section.Clear;
      ini.ReadSectionValues(sections[i], section);
      with section do
        if IrcOrHub = 'hubs' then
        begin
          Hubs.AddHub(sections[i], Values['Nick'], Values['Pass'], Values['Prefix'], Values['Share'], Values['Slots'], Values['Client'], Values['Enabled'], Values['BroadcastFrom'], Values['BroadcastTo']);
          UpdateDisplayHubs;
        end
        else
        begin
          Ircs.AddIrc(sections[i], Values['Nick'], Values['Pass'], Values['Prefix'], Values['Ident'], Values['Description'], Values['Channels'], Values['Enabled'], Values['BroadcastFrom'], Values['BroadcastTo']);
          UpdateDisplayIrcs;
        end;
    end;
  finally
    section.Free;
    sections.Free;
    ini.Free;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.SaveIrcsHubs(IrcOrHub: string);
var
  fname, s: string;
  i: integer;
  ini: TIniFile;
begin
  IrcOrHub := LowerCase(IrcOrHub);
  if IrcOrHub = 'hubs' then
    fname := BaseDir + FILE_HUBS
  else
    fname := BaseDir + FILE_IRCS;

  DeleteFile(fname);
  ini := TIniFile.Create(fname);
  try
    if IrcOrHub = 'hubs' then
      for i := 0 to Hubs.Count-1 do
        with Hubs.Items[i] do
        begin
          s := Format('%s:%d', [Host, Port]);
          ini.WriteString(s, 'Nick', Nick);
          ini.WriteString(s, 'Pass', Pass);
          ini.WriteString(s, 'Prefix', Prefix);
          ini.WriteString(s, 'Share', Share);
          ini.WriteString(s, 'Slots', Slots);
          ini.WriteString(s, 'Client', Client);
          ini.WriteString(s, 'Enabled', IntToStr(Ord(Enabled)));
          ini.WriteString(s, 'BroadcastFrom', IntToStr(Ord(BcFrom)));
          ini.WriteString(s, 'BroadcastTo', IntToStr(Ord(BcTo)));
        end
    else
      for i := 0 to Ircs.Count-1 do
      with Ircs.Items[i] do
        begin
          s := Format('%s:%d', [Host, Port]);
          ini.WriteString(s, 'Nick', Nick);
          ini.WriteString(s, 'Pass', Pass);
          ini.WriteString(s, 'Prefix', Prefix);
          ini.WriteString(s, 'Ident', Ident);
          ini.WriteString(s, 'Description', Descr);
          ini.WriteString(s, 'Channels', Channels);
          ini.WriteString(s, 'Enabled', IntToStr(Ord(Enabled)));
          ini.WriteString(s, 'BroadcastFrom', IntToStr(Ord(BcFrom)));
          ini.WriteString(s, 'BroadcastTo', IntToStr(Ord(BcTo)));
        end;
  finally
    ini.Free;
  end;
end;
//------------------------------------------------------------------------------
function StrToBool(s: string): boolean;
begin
  s := LowerCase(s);
  Result := (s = 'on') or (s = '1') or (s = 'yes');
end;
//------------------------------------------------------------------------------
procedure TMainForm.lvHubsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  hub: THub;
begin
  if Selected and (Item <> nil) and (Item.Data <> nil) then begin
    hub := Item.Data;
    edHubHostPort.Text := hub.Host + ':' + IntToStr(hub.Port);
    edHubNick.Text := hub.Nick;
    edHubPass.Text := hub.Pass;
    edHubPrefix.Text := hub.Prefix;
    edHubShare.Text := hub.Share;
    edHubSlots.Text := hub.Slots;
    edHubClient.Text := hub.Client;
    cbHubBcFrom.Checked := hub.BcFrom;
    cbHubBcTo.Checked := hub.BcTo;
    cbHubEnabled.Checked := hub.Enabled;
    btnHubUpdate.Enabled := true;
    btnHubDelete.Enabled := true;
  end else begin
    edHubHostPort.Clear;
    edHubNick.Clear;
    edHubPass.Clear;
    edHubPrefix.Clear;
    edHubShare.Clear;
    edHubSlots.Clear;
    edHubClient.Clear;
    cbHubBcFrom.Checked := false;
    cbHubBcTo.Checked := false;
    cbHubEnabled.Checked := false;
    btnHubUpdate.Enabled := false;
    btnHubDelete.Enabled := false;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.lvIrcsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  irc: TIrc;
begin
  if Selected and (Item <> nil) and (Item.Data <> nil) then begin
    irc := Item.Data;
    edIrcHostPort.Text := irc.Host + ':' + IntToStr(irc.Port);
    edIrcNick.Text := irc.Nick;
    edIrcPass.Text := irc.Pass;
    edIrcPrefix.Text := irc.Prefix;
    edIrcIdent.Text := irc.Ident;
    edIrcDescription.Text := irc.Descr;
    edIrcChannels.Text := irc.Channels;
    cbIrcBcFrom.Checked := irc.BcFrom;
    cbIrcBcTo.Checked := irc.BcTo;
    cbIrcEnabled.Checked := irc.Enabled;
    btnIrcUpdate.Enabled := true;
    btnIrcDelete.Enabled := true;
  end else begin
    edIrcHostPort.Clear;
    edIrcNick.Clear;
    edIrcPass.Clear;
    edIrcPrefix.Clear;
    edIrcIdent.Clear;
    edIrcDescription.Clear;
    edIrcChannels.Clear;
    cbIrcBcFrom.Checked := false;
    cbIrcBcTo.Checked := false;
    cbIrcEnabled.Checked := false;
    btnIrcUpdate.Enabled := false;
    btnIrcDelete.Enabled := false;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnHubUpdateClick(Sender: TObject);
var
  hub: THub;
begin
  if (lvHubs.Selected <> nil) and (lvHubs.Selected.Data <> nil) then
  begin
    hub := lvHubs.Selected.Data;
    GetHostPort(edHubHostPort.Text, hub.Host, hub.Port, 411);
    hub.Nick := edHubNick.Text;
    hub.Pass := edHubPass.Text;
    hub.Prefix := edHubPrefix.Text;
    hub.Share := edHubShare.Text;
    hub.Slots := edHubSlots.Text;
    hub.Client := edHubClient.Text;
    hub.BcFrom := cbHubBcFrom.Checked;
    hub.BcTo := cbHubBcTo.Checked;
    hub.Enabled := cbHubEnabled.Checked;
    UpdateDisplayHubs;
 end
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnIrcUpdateClick(Sender: TObject);
var
  irc: TIrc;
begin
  if (lvIrcs.Selected <> nil) and (lvIrcs.Selected.Data <> nil) then
  begin
    irc := lvIrcs.Selected.Data;
    GetHostPort(edIrcHostPort.Text, irc.Host, irc.Port, 411);
    irc.Nick := edIrcNick.Text;
    irc.Pass := edIrcPass.Text;
    irc.Prefix := edIrcPrefix.Text;
    irc.Ident := edIrcIdent.Text;
    irc.Descr := edIrcDescription.Text;
    if (irc.Channels <> edIrcChannels.Text) and irc.Enabled then
    begin
      irc.Tcp.Socket.SendText(Format('PART %s'#13#10, [irc.Channels]));
      irc.Channels := edIrcChannels.Text;
      irc.Tcp.Socket.SendText(Format('JOIN %s'#13#10, [irc.Channels]));
    end;
    irc.Channels := edIrcChannels.Text;
    irc.BcFrom := cbIrcBcFrom.Checked;
    irc.BcTo := cbIrcBcTo.Checked;
    irc.Enabled := cbIrcEnabled.Checked;
    UpdateDisplayIrcs;
 end
end;
//------------------------------------------------------------------------------
procedure TMainForm.TimerPollTimer(Sender: TObject);
begin
  ConnectAll;
end;
//------------------------------------------------------------------------------
procedure TMainForm.ConnectAll;
var
  i: integer;
begin
  for i := 0 to Hubs.Count-1 do
    if not Hubs[i].Connected and Hubs[i].Enabled then
      Hubs[i].Connect;

  for i := 0 to Ircs.Count-1 do
    if not Ircs[i].Connected and Ircs[i].Enabled then
      Ircs[i].Connect;
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnConnectAllClick(Sender: TObject);
begin
  ConnectAll;
end;
//------------------------------------------------------------------------------
procedure TMainForm.Broadcast(Origin: Pointer; Prefix, ANick, Msg, MsgFormat: string);
var
  i: integer;
  prfx: string;
begin
  if (Trim(Msg) = '') or HasBadWords(Msg) then
    Exit;

  Msg := StringReplace(Msg, #$02, '', [rfReplaceAll]);
  Msg := StringReplace(Msg, #$03, '', [rfReplaceAll]);
  Msg := StringReplace(Msg, #$16, '', [rfReplaceAll]);
  Msg := StringReplace(Msg, #$1F, '', [rfReplaceAll]);

  EnterCriticalSection(CSNotification);
  try
    for i := 0 to Hubs.Count-1 do
      if Hubs[i] <> Origin then
        if Hubs[i].Enabled and Hubs[i].BcTo and not Hubs[i].Muted and not Hubs[i].NickIgnored(Prefix, ANick) then
        begin
          prfx := Hubs[i].GetPrefixedNick(Prefix, ANick);
          Hubs[i].SendMsgToChat(Format(MsgFormat, [prfx, Msg]));
        end;
    for i := 0 to Ircs.Count-1 do
      if Ircs[i] <> Origin then
        if Ircs[i].Enabled and Ircs[i].BcTo and not Ircs[i].Muted and not Ircs[i].NickIgnored(Prefix, ANick) then
        begin
          prfx := Ircs[i].GetPrefixedNick(Prefix, ANick);
          Ircs[i].SendMsgToChat(Format(MsgFormat, [prfx, Msg]));
        end;
  finally
    LeaveCriticalSection(CSNotification);
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.RefreshItem(Origin: Pointer);
var
  i: integer;
begin
  EnterCriticalSection(CSRefreshItem);
  try
    for i := 0 to lvHubs.Items.Count-1 do
      if lvHubs.Items[i].Data = Origin then
      begin
        lvHubs.Items[i].ImageIndex := Ord(THub(Origin).Connected);
        Exit;
      end;

    for i := 0 to lvIrcs.Items.Count-1 do
      if lvIrcs.Items[i].Data = Origin then
      begin
        lvIrcs.Items[i].ImageIndex := Ord(TIrc(Origin).Connected);
        Exit;
      end;
  finally
    LeaveCriticalSection(CSRefreshItem);
  end;
end;
//------------------------------------------------------------------------------
function TMainForm.GetLinks: string;
var
  i: integer;
  hh, ii: string;
  hc, ic: integer;
begin
  Result := '';
  hh := ''; ii := '';
  hc := 0; ic := 0;
  EnterCriticalSection(CSGetLinks);
  try
    for i := 0 to Hubs.Count-1 do
      if Hubs[i].Enabled then
      begin
        hh := hh + Format('%s (%s),', [Hubs[i].Prefix, StringReplace(Hubs[i].Host, '.', '_', [rfReplaceAll])]);
        Inc(hc);
      end;
    Delete(hh, Length(hh), 1); // trailing ,

    for i := 0 to Ircs.Count-1 do
      if Ircs[i].Enabled then
      begin
        ii := ii + Format('%s (%s %s),', [Ircs[i].Prefix, StringReplace(Ircs[i].Host, '.', '_', [rfReplaceAll]), Ircs[i].Channels]);
        Inc(ic);
      end;
    if Ircs.Count > 0 then
      Delete(ii, Length(Result), 1);

    Result := Format('Hubs(%d): %s. Ircs(%d): %s', [hc, hh, ic, ii]);
  finally
    LeaveCriticalSection(CSGetLinks);
  end;
end;
//------------------------------------------------------------------------------
function TMainForm.HasBadWords(msg: string): boolean;
var
  i: integer;
begin
  Result := true;
  msg := LowerCase(msg);
  for i := 0 to BadWords.Count-1 do
    if Pos(LowerCase(BadWords[i]), msg) > 0 then
      Exit;
  Result := false;
end;
//------------------------------------------------------------------------------
procedure TMainForm.LoadBadWords;
begin
  if FileExists(BaseDir + FILE_BADWORDS) then
    BadWords.LoadFromFile(BaseDir + FILE_BADWORDS);
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnHubNewClick(Sender: TObject);
begin
  Hubs.AddHub('new.dc.hub', 'default_nick', 'default_pass', 'default_prefix', '99999999', '5', 'ApexDC++ V:1.2.1', '1', '1', '1');
  UpdateDisplayHubs;
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnIrcNewClick(Sender: TObject);
begin
  Ircs.AddIrc('new.irc.server', 'default_nick', 'default_pass', 'default_prefix', 'MyIrcDcLinks', MSG_VERSION, '#chan1,#chan2,#chan3', '1', '1', '1');
  UpdateDisplayIrcs;
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnHubDeleteClick(Sender: TObject);
begin
  if lvHubs.Selected = nil then
    Exit;
  Hubs.DeleteHub(lvHubs.Selected.Data);
  UpdateDisplayHubs;
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnIrcDeleteClick(Sender: TObject);
begin
  if lvIrcs.Selected = nil then
    Exit;
  Ircs.DeleteIrc(lvIrcs.Selected.Data);
  UpdateDisplayIrcs;
end;
//------------------------------------------------------------------------------
procedure TMainForm.UpdateDisplayHubs;
var
  i: integer;
  p: Pointer;
begin
  lvHubs.Items.BeginUpdate;
  p := nil;
  if lvHubs.Selected <> nil then
    p := lvHubs.Selected.Data;
  try
    lvHubs.Clear;
    for i := 0 to Hubs.Count-1 do
      with lvHubs.Items.Add do
      begin
        Data := Hubs[i];
        Caption := Format('%s:%d', [Hubs[i].Host, Hubs[i].Port]);
        SubItems.Add(Hubs[i].Prefix);
        SubItems.Add(Hubs[i].Nick);
        SubItems.Add(Hubs[i].Share);
        SubItems.Add(Hubs[i].Slots);
        SubItems.Add(Hubs[i].Client);
        ImageIndex := Ord(Hubs[i].Connected);
      end;
  finally
    if p <> nil then
      for i := 0 to lvHubs.Items.Count-1 do
        if lvHubs.Items[i].Data = p then
          lvHubs.Selected := lvHubs.Items[i];
    lvHubs.Items.EndUpdate;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.UpdateDisplayIrcs;
var
  i: integer;
  p: Pointer;
begin
  lvIrcs.Items.BeginUpdate;
  p := nil;
  if lvIrcs.Selected <> nil then
    p := lvIrcs.Selected.Data;
  try
    lvIrcs.Clear;
    for i := 0 to Ircs.Count-1 do
      with lvIrcs.Items.Add do
      begin
        Data := Ircs[i];
        Caption := Format('%s:%d', [Ircs[i].Host, Ircs[i].Port]);
        SubItems.Add(Ircs[i].Prefix);
        SubItems.Add(Ircs[i].Nick);
        SubItems.Add(Ircs[i].Ident);
        SubItems.Add(Ircs[i].Descr);
        SubItems.Add(Ircs[i].Channels);
        ImageIndex := Ord(Ircs[i].Connected); 
      end;
  finally
    if p <> nil then
      for i := 0 to lvIrcs.Items.Count-1 do
        if lvIrcs.Items[i].Data = p then
          lvIrcs.Selected := lvIrcs.Items[i];
    lvIrcs.Items.EndUpdate;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.pmConnDisconnPopup(Sender: TObject);
var
  lv: TListView;
begin
  if not (pmConnDisconn.PopupComponent is TListView) then
    Exit;

  lv := TListView(pmConnDisconn.PopupComponent);
  Connect1.Enabled := lv.Selected <> nil;
  Disconnect1.Enabled := Connect1.Enabled;
end;
//------------------------------------------------------------------------------
procedure TMainForm.Connect1Click(Sender: TObject);
var
  lv: TListView;
begin
  if not (pmConnDisconn.PopupComponent is TListView) then
    Exit;

  lv := TListView(pmConnDisconn.PopupComponent);
  if (lv.Selected = nil) or (lv.Selected.Data = nil) then
    Exit;

  if lv = lvHubs then
    THub(lv.Selected.Data).Connect
  else if lv = lvIrcs then
    TIrc(lv.Selected.Data).Connect;
end;
//------------------------------------------------------------------------------
procedure TMainForm.Disconnect1Click(Sender: TObject);
var
  lv: TListView;
begin
  if not (pmConnDisconn.PopupComponent is TListView) then
    Exit;

  lv := TListView(pmConnDisconn.PopupComponent);
  if (lv.Selected = nil) or (lv.Selected.Data = nil) then
    Exit;

  if lv = lvHubs then
    THub(lv.Selected.Data).Tcp.Close
  else if lv = lvIrcs then
    TIrc(lv.Selected.Data).Tcp.Close;
end;
//------------------------------------------------------------------------------
function TMainForm.HasLinkPrefix(anick: string): boolean;
var
  i: integer;
begin
  Result := true;
  for i := 0 to Hubs.Count-1 do
    if Pos(Hubs[i].GetPrefixedNick(Hubs[i].Prefix, ''), anick) = 1 then
      Exit;
  for i := 0 to Ircs.Count-1 do
    if Pos(Ircs[i].GetPrefixedNick(Ircs[i].Prefix, ''), anick) = 1 then
      Exit;
  Result := false;
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnBadWordsSaveClick(Sender: TObject);
begin
  mmBadWords.Lines.SaveToFile(BaseDir + FILE_BADWORDS);
end;
//------------------------------------------------------------------------------
procedure TMainForm.btnBadWordsReloadClick(Sender: TObject);
begin
  LoadBadWords;
  mmBadWords.Lines.Assign(BadWords);
end;
//------------------------------------------------------------------------------
procedure TMainForm.cbAutoConnectClick(Sender: TObject);
begin
  TimerPoll.Enabled := cbAutoConnect.Checked;
end;
//------------------------------------------------------------------------------
end.
